package ru.dbotthepony.mc.otm.data

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.codecs.RecordCodecBuilder
import it.unimi.dsi.fastutil.objects.ObjectArrayList
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.storage.loot.LootContext
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition
import net.minecraftforge.common.loot.IGlobalLootModifier
import net.minecraftforge.common.loot.LootModifier
import java.util.Arrays
import java.util.stream.Stream

class BasicLootAppender(
	conditions: Array<out LootItemCondition>,
	items: Stream<ItemStack>
) : LootModifier(conditions) {
	constructor(
		conditions: Array<out LootItemCondition>,
		items: Collection<ItemStack>
	) : this(conditions, items.stream())

	constructor(
		conditions: Array<out LootItemCondition>,
		vararg items: ItemStack
	) : this(conditions, Arrays.stream(items))

	private val items = items.map { it.copy() }.collect(ImmutableList.toImmutableList())

	override fun codec(): Codec<out IGlobalLootModifier> {
		return CODEC
	}

	override fun doApply(
		generatedLoot: ObjectArrayList<ItemStack>,
		context: LootContext
	): ObjectArrayList<ItemStack> {
		for (item in items)
			generatedLoot.add(item.copy())

		return generatedLoot
	}

	companion object {
		val CODEC: Codec<BasicLootAppender> by lazy {
			RecordCodecBuilder.create {
				codecStart(it)
					.and(ItemStackCodec.LIST.fieldOf("items").forGetter(BasicLootAppender::items))
					.apply(it, ::BasicLootAppender)
			}
		}
	}
}
